
#ifndef PLS_INTERNAL_PROFILING_THREAD_STATS_H_
#define PLS_INTERNAL_PROFILING_THREAD_STATS_H_

#include <chrono>
#include <type_traits>

#include "context_switcher/context_switcher.h"

#include "pls/internal/base/system_details.h"
#include "pls/internal/base/stack_allocator.h"

namespace pls::internal::profiling {

struct PLS_CACHE_ALIGN thread_stats {
  static constexpr size_t STACK_SIZE = 4096 * 4;

  thread_stats() {
    stack_ = stack_allocator_.allocate_stack(STACK_SIZE);
  }
  ~thread_stats() {
    stack_allocator_.free_stack(STACK_SIZE, stack_);
  }

  template<typename Function>
  void run_on_stack(const Function function) {
    context_switcher::enter_context(stack_, STACK_SIZE, [function](auto cont) {
      function();
      return std::move(cont);
    });
  }

  using clock = std::chrono::steady_clock;

  unsigned long total_steals_{0};
  unsigned long successful_steals_{0};
  unsigned long failed_steals_{0};
  unsigned long steal_cas_ops_{0};

  unsigned long total_time_stealing_{0};

  clock::time_point stealing_start_time_;
  clock::time_point task_run_start_time;

  base::mmap_stack_allocator stack_allocator_;
  char *stack_;
};

}

#endif //PLS_INTERNAL_PROFILING_THREAD_STATS_H_
